@@ -7,11 +7,12 @@ static char *family_strings[] = {"Unknown", "legacy", "radeon",
 				 "rs480", "rv515", "r520", "rv530", "r580",
 				 "rv560", "rv570", "rs600", "rs690", "rs740",
 				 "r600", "r630", "rv610", "rv630", "rv670",
-				 "rv620", "rv635", "rs780", "rs880", 
-				"rv770", "rv730", "rv710", "rv740",
-				"cedar", "redwood", "juniper",
-				"cypress", "hemlock" };
-			  
+				 "rv620", "rv635", "rs780", "rs880",
+				 "rv770", "rv730", "rv710", "rv740",
+				 "cedar", "redwood", "juniper",
+				 "cypress", "hemlock",
+				 "palm",
+				 "barts", "turks", "caicos" };
 
 typedef enum {
     CHIP_FAMILY_UNKNOW,
@@ -20,8 +21,7 @@ typedef enum {
     CHIP_FAMILY_RV100,
     CHIP_FAMILY_RS100,    /* U1 (IGP320M) or A3 (IGP320)*/
     CHIP_FAMILY_RV200,
-    CHIP_FAMILY_RS200,    /* U2 (IGP330M/340M/350M) or A4 (IGP330/340/345/350), 
-RS250 (IGP 7000) */
+    CHIP_FAMILY_RS200,    /* U2 (IGP330M/340M/350M) or A4 (IGP330/340/345/350), RS250 (IGP 7000) */
     CHIP_FAMILY_R200,
     CHIP_FAMILY_RV250,
     CHIP_FAMILY_RS300,    /* RS300/RS350 */
@@ -61,6 +61,11 @@ RS250 (IGP 7000) */
     CHIP_FAMILY_JUNIPER,
     CHIP_FAMILY_CYPRESS,
     CHIP_FAMILY_HEMLOCK,
+    CHIP_FAMILY_PALM,
+    CHIP_FAMILY_BARTS,
+    CHIP_FAMILY_TURKS,
+    CHIP_FAMILY_CAICOS,
+    CHIP_FAMILY_CAYMAN,
     CHIP_FAMILY_LAST
 } RADEONChipFamily;
 
@@ -78,3 +83,19 @@ typedef struct {
 
 #define IS_RV620(card_info) (card_info && card_info->chip_family >= CHIP_FAMILY_RV620)
 #define IS_EG(card_info) (card_info && card_info->chip_family >= CHIP_FAMILY_CEDAR)
+
+#define IS_DISPLAY_RADEON(card_info) (card_info && \
+				      (card_info->chip_family >= CHIP_FAMILY_RADEON) && \
+				      (card_info->chip_family <= CHIP_FAMILY_RS480))
+#define IS_DISPLAY_AVIVO(card_info) (card_info && \
+				     (card_info->chip_family >= CHIP_FAMILY_RV515) && \
+				     (card_info->chip_family <= CHIP_FAMILY_RV670))
+#define IS_DISPLAY_DCE3(card_info) (card_info && \
+				    (card_info->chip_family >= CHIP_FAMILY_RV620) && \
+				    (card_info->chip_family <= CHIP_FAMILY_RV740))
+#define IS_DISPLAY_DCE4(card_info) (card_info && \
+				    (card_info->chip_family >= CHIP_FAMILY_CEDAR) && \
+				    (card_info->chip_family <= CHIP_FAMILY_PALM))
+#define IS_DISPLAY_DCE5(card_info) (card_info && \
+				    (card_info->chip_family >= CHIP_FAMILY_BARTS) && \
+				    (card_info->chip_family <= CHIP_FAMILY_CAICOS))
