@@ -16,6 +16,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,15 +32,20 @@ int skip = 0;
 
 /* *radeon_cntl_mem is mapped to the actual device's memory mapped control area. */
 /* Not the address but what it points to is volatile. */
-struct pci_device *avivo_device = NULL;
 unsigned char * volatile radeon_cntl_mem;
-int ctrl_region = -1, fb_region = -1;
 unsigned char * volatile fb_mem;
 unsigned char * volatile ctrl_mem;
 static void radeon_rom_legacy_mmio_table(unsigned char *bios, int offset);
-static void fatal(char *why)
+static void die(const char *why)
 {
-    fprintf(stderr,why);
+    fprintf(stderr, "fatal error: %s\n", why);
+    pci_system_cleanup();
+    exit(-1);
+}
+
+static void die_error(int err, const char *why)
+{
+    fprintf(stderr, "fatal error: %s: %s\n", why, strerror(err));
     pci_system_cleanup();
     exit(-1);
 }
@@ -49,10 +55,7 @@ static unsigned int radeon_get(unsigned long offset, const char *name)
     unsigned int value;
     if(debug) 
         printf("reading %s (%lx) is ",name,offset);
-    if(radeon_cntl_mem == NULL) {
-        printf("internal error\n");
-	exit(-2);
-    };
+    assert(radeon_cntl_mem);
 #ifdef __powerpc__
     __asm__ __volatile__ ("lwbrx %0,%1,%2\n\t"
 			  "eieio"
@@ -70,10 +73,7 @@ static void radeon_set(unsigned long offset, const char *name, unsigned int valu
 {
     if(debug) 
         printf("writing %s (%lx) -> %08x\n",name,offset,value);
-    if(radeon_cntl_mem == NULL) {
-        printf("internal error\n");
-	exit(-2);
-    };
+    assert(radeon_cntl_mem);
 #ifdef __powerpc__
     __asm__ __volatile__ ("stwbrx %1,%2,%3\n\t"
 			  "eieio"
@@ -91,6 +91,7 @@ static unsigned int radeon_get_indexed(unsigned long index_offset,
 				       unsigned long index_mask)
 {
     unsigned int temp = offset;
+    assert(radeon_cntl_mem);
     if (index_mask) {
 	temp = radeon_get(index_offset, "index");
 	temp &= ~index_mask;
@@ -105,6 +106,7 @@ static void radeon_set_indexed(unsigned long index_offset,
                                const char *name, unsigned int value, int index_mask)
 {
     unsigned int temp = offset;
+    assert(radeon_cntl_mem);
     if (index_mask) {
 	temp = radeon_get(index_offset, "index");
 	temp &= ~index_mask;
@@ -141,13 +143,25 @@ static void radeon_set_mcind(unsigned long offset, const char *name,
 
 static void usage(void)
 {
+    const char *dac_status, *light_status;
+
+    if (!radeon_cntl_mem) {
+        dac_status = "";
+        light_status = "";
+    } else {
+        dac_status = (radeon_get(RADEON_DAC_CNTL, "RADEON_DAC_CNTL") & RADEON_DAC_PDWN)
+            ? " (off)" : " (on)";
+        light_status = (radeon_get(RADEON_LVDS_GEN_CNTL,"RADEON_LVDS_GEN_CNTL") & RADEON_LVDS_ON)
+            ? " (on)" : " (off)";
+    }
+
     fprintf(stderr,"usage: radeontool [options] [command]\n");
     fprintf(stderr,"         --debug            - show a little debug info\n");
     fprintf(stderr,"         --skip=1           - use the second radeon card\n");
-    fprintf(stderr,"         dac [on|off]       - power down the external video outputs (%s)\n",
-	   (radeon_get(RADEON_DAC_CNTL,"RADEON_DAC_CNTL")&RADEON_DAC_PDWN)?"off":"on");
-    fprintf(stderr,"         light [on|off]     - power down the backlight (%s)\n",
-	   (radeon_get(RADEON_LVDS_GEN_CNTL,"RADEON_LVDS_GEN_CNTL")&RADEON_LVDS_ON)?"on":"off");
+    fprintf(stderr,"         dac [on|off]       - power down the external video outputs%s\n",
+	   dac_status);
+    fprintf(stderr,"         light [on|off]     - power down the backlight%s\n",
+	   light_status);
     fprintf(stderr,"         stretch [on|off|vert|horiz|auto|manual]   - stretching for resolution mismatch \n");
     fprintf(stderr,"         regs               - show a listing of some random registers\n");
     fprintf(stderr,"         regmatch <pattern> - show registers matching wildcard pattern\n");
@@ -159,6 +173,8 @@ void radeon_cmd_regs(void)
 {
 #define SHOW_REG(r) printf("%s\t%08x\n", #r, radeon_get(r, #r))
 #define SHOW_CLK_REG(r) printf("%s\t%08x\n", #r, radeon_get_indexed(RADEON_CLOCK_CNTL_INDEX, RADEON_CLOCK_CNTL_DATA, (r), #r, 0x3f))	
+	if (!radeon_cntl_mem)
+		exit(-1);
 	SHOW_REG(RADEON_DAC_CNTL);
 	SHOW_REG(RADEON_DAC_EXT_CNTL);
 	SHOW_REG(RADEON_DAC_MACRO_CNTL);
@@ -722,6 +738,8 @@ void radeon_reg_match(const char *pattern)
     unsigned long address;
     unsigned int value;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     if (pattern[0] == '0' && pattern[1] == 'x') {
         address = strtol(&(pattern[2]), NULL, 16);
         value = radeon_get(address, pattern);
@@ -796,6 +814,8 @@ void radeon_reg_set(const char *inname, unsigned int value)
     int i;
     unsigned long address;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     if (inname[0] == '0' && inname[1] == 'x') {
         address = strtol(&(inname[2]), NULL, 16);
         set_reg(inname, "", address, value, radeon_get, radeon_set);
@@ -818,6 +838,8 @@ void radeon_cmd_bits(void)
 {
     unsigned int dac_cntl;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     dac_cntl = radeon_get(RADEON_DAC_CNTL,"RADEON_DAC_CNTL");  
     printf("RADEON_DAC_CNTL=%08x (",dac_cntl);  
     if(dac_cntl & RADEON_DAC_RANGE_CNTL)
@@ -837,6 +859,8 @@ void radeon_cmd_dac(char *param)
 {
     unsigned long dac_cntl;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     dac_cntl = radeon_get(RADEON_DAC_CNTL,"RADEON_DAC_CNTL");
     if(param == NULL) {
         printf("The radeon external DAC looks %s\n",(dac_cntl&(RADEON_DAC_PDWN))?"off":"on");
@@ -855,6 +879,8 @@ void radeon_cmd_light(char *param)
 {
     unsigned long lvds_gen_cntl;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     lvds_gen_cntl = radeon_get(RADEON_LVDS_GEN_CNTL,"RADEON_LVDS_GEN_CNTL");
     if(param == NULL) {
         printf("The radeon backlight looks %s\n",(lvds_gen_cntl&(RADEON_LVDS_ON))?"on":"off");
@@ -873,6 +899,8 @@ void radeon_cmd_stretch(char *param)
 {
     unsigned long fp_vert_stretch,fp_horz_stretch;
 
+    if (!radeon_cntl_mem)
+        exit(-1);
     fp_vert_stretch = radeon_get(RADEON_FP_VERT_STRETCH,"RADEON_FP_VERT_STRETCH");
     fp_horz_stretch = radeon_get(RADEON_FP_HORZ_STRETCH,"RADEON_FP_HORZ_STRETCH");
     if(param == NULL) {
@@ -917,10 +945,15 @@ static void map_radeon_cntl_mem(void)
     struct pci_slot_match match;
     struct pci_device_iterator *iter;
     struct pci_device *device;
-    int i = 0;
+    pciaddr_t fb_base, fb_size, ctrl_base, ctrl_size;
+    int i = 0, ret;
 
-    if (pci_system_init() != 0)
-        fatal("failed to initialise libpciaccess\n");
+    ret = pci_system_init();
+    if (ret) {
+        fprintf(stderr, "error: failed to initialise libpciaccess: %s\n",
+		strerror(ret));
+        return;
+    }
 
     match.domain = PCI_MATCH_ANY;
     match.bus = PCI_MATCH_ANY;
@@ -932,48 +965,42 @@ static void map_radeon_cntl_mem(void)
 
     while ((device = pci_device_next(iter))) {
         pci_device_probe(device);
-        if (device->vendor_id == 0x1002 &&
-            (device->device_class & 0x00ffff00) == 0x00030000) {
-            if (debug) {
-                printf("Found card %x:%x (%x)\n", device->vendor_id,
-                       device->device_id, device->device_class);
-            }
+        if (device->vendor_id != 0x1002)
+            continue;
+        if ((device->device_class & 0x00ffff00) != 0x00030000)
+            continue;
 
-            if (skip-- != 0) {
-                continue;
-            }
-            for (i = 0; i < 6; i++) {
-                if (device->regions[i].size >= 16 * 1024 &&
-                    device->regions[i].size <= 64 * 1024) {
-                    if (ctrl_region != -1)
-                        fatal("cannot distinguish ctrl region\n");
-                    ctrl_region = i;
-                } else if (device->regions[i].size >= 64 * 1024 * 1024) {
-                    if (fb_region != -1)
-                        fatal("cannot distinguish fb region\n");
-                    fb_region = i;
-                }
-            }
-            if (ctrl_region == -1)
-                fatal("cannot find ctrl region\n");
-            if (fb_region == -1)
-                fatal("cannot find fb region\n");
-            avivo_device = device;
-            break;
-        }
+        if (debug)
+            printf("Found card %x:%x (%x)\n", device->vendor_id,
+                   device->device_id, device->device_class);
+        if (skip--)
+            continue;
+        break;
     }
 
-    if (!avivo_device)
-        fatal("cannot find Radeon device\n");
+    if (!device) {
+        fprintf(stderr, "error: cannot find Radeon device\n");
+        return;
+    }
 
-    if (pci_device_map_region(avivo_device, ctrl_region, 1) != 0)
-        fatal("mapping ctrl region\n");
-    ctrl_mem = avivo_device->regions[ctrl_region].memory;
+    ctrl_base = device->regions[2].base_addr;
+    ctrl_size = device->regions[2].size;
+    if (!ctrl_size) {
+        fprintf(stderr, "error: missing ctrl region\n");
+        return;
+    }
+    ret = pci_device_map_range(device, ctrl_base, ctrl_size,
+			     PCI_DEV_MAP_FLAG_WRITABLE, (void **) &ctrl_mem);
+    if (ret) {
+        fprintf(stderr, "cannot map ctrl region: %s\n", strerror(ret));
+        return;
+    }
 
-    if (pci_device_map_region(avivo_device, fb_region, 1) == 0)
-    	fb_mem = avivo_device->regions[fb_region].memory;
-    else
-	fb_mem = NULL;
+    fb_base = device->regions[0].base_addr;
+    fb_size = device->regions[0].size;
+    if (!fb_size || pci_device_map_range(device, fb_base, fb_size,
+			     PCI_DEV_MAP_FLAG_WRITABLE, (void **) &fb_mem))
+        fb_mem = NULL;
 
     pci_iterator_destroy(iter);
 
@@ -982,6 +1009,8 @@ static void map_radeon_cntl_mem(void)
                "base framebuffer address is %lx.\n",
                (unsigned long) ctrl_mem, (unsigned long) fb_mem);
 
+    if (!ctrl_mem)
+        die("internal error");
     radeon_cntl_mem = ctrl_mem;
 }
 
@@ -2788,6 +2817,81 @@ void radeon_rom_legacy_memclktable(unsigned char *bios, int hdr)
 		block_rev = BIOS8(offset);
 	}
 }
+
+#define OF32(offset)	(*((unsigned int *)(bios + (offset))))
+#define _2K (2*1024)
+
+#define OF_TABLE_INIT1 1
+#define OF_TABLE_INIT2 2
+#define OF_TABLE_PLL 3
+#define OF_TABLE_MEMINIT 4
+
+void radeon_of_table(const char * file, int table)
+{
+	unsigned char bios[_2K];
+	unsigned int reg, mask, val, op;
+	int fd;
+	int len;
+	int count = 0;
+	fd = open(file, O_RDONLY);
+	if (fd < 0) {
+		perror("can't open init file");
+		return;
+	}
+	memset(bios, 0, _2K);
+	len = read(fd, bios, _2K);
+	close(fd);
+
+	while (len > 0) {
+		switch(table) {
+		case OF_TABLE_INIT1:
+			len -= 12;
+			reg = OF32(count);
+			mask = OF32(count + 4);
+			val = OF32(count + 8);
+			printf("reg: 0x%x, mask 0x%x, val 0x%x\n", reg, mask, val);
+			count += 12;
+			break;
+		case OF_TABLE_INIT2:
+			len -= 16;
+			reg = OF32(count);
+			mask = OF32(count + 4);
+			val = OF32(count + 8);
+			op = OF32(count + 12);
+			printf("reg: 0x%x, mask 0x%x, val 0x%x op 0x%x\n", reg, mask, val, op);
+			count += 16;
+			break;
+		case OF_TABLE_PLL:
+			len -= 16;
+			reg = OF32(count);
+			mask = OF32(count + 4);
+			val = OF32(count + 8);
+			op = OF32(count + 12);
+			printf("pll reg: 0x%x, mask 0x%x, val 0x%x, op %x\n", reg, mask, val, op);
+			if (op == 5)
+				printf("delay 1 ms\n");
+			else if (op == 6)
+				printf("delay 150 us\n");
+			else if (op)
+				printf("unknown op %d\n", op);
+			count += 16;
+			break;
+		case OF_TABLE_MEMINIT:
+			len -= 4;
+			val = OF32(count);
+			if (val >> 16 == 0x6fff)
+				printf("pwup complete test\n");
+			else {
+				mask = val >> 24;
+				val = val & 0xffff;
+				printf("rmw SDRAM mask %x %x\n", val, mask);
+			}
+			count += 4;
+			break;
+		}
+	}
+}
+
 void radeon_rom_tables(const char * file)
 {
 #define _64K (64*1024)
@@ -2804,7 +2908,7 @@ void radeon_rom_tables(const char * file)
 	close(fd);
 
 	if (bios[0] != 0x55 || bios[1] != 0xaa)
-		fatal("PCI ROM signature 0x55 0xaa missing");
+		die("PCI ROM signature 0x55 0xaa missing");
 	hdr = BIOS16(0x48);
 	printf("\nBIOS Tables:\n------------\n\n");	
 	printf("Header at %x, type: %d [%s]\n", hdr, BIOS8(hdr),
@@ -2876,10 +2980,6 @@ void radeon_rom_tables(const char * file)
 
 int main(int argc,char *argv[]) 
 {
-    if(argc == 1) {
-        map_radeon_cntl_mem();
-	usage();
-    }
     while (argc >= 2) {
         if(strcmp(argv[1],"--debug") == 0) {
             debug=1;
@@ -2891,8 +2991,8 @@ int main(int argc,char *argv[])
             break;
         }
     }
-    map_radeon_cntl_mem();
     if(argc == 2) {
+    	map_radeon_cntl_mem();
         if(strcmp(argv[1],"regs") == 0) {
             radeon_cmd_regs();
             return 0;
@@ -2910,6 +3010,27 @@ int main(int argc,char *argv[])
             return 0;
         };
     } else if(argc == 3) {
+        if(strcmp(argv[1],"romtables") == 0) {
+		radeon_rom_tables(argv[2]);
+		return 0;
+	}
+        if(strcmp(argv[1],"oftablepll") == 0) {
+		radeon_of_table(argv[2], OF_TABLE_PLL);
+		return 0;
+	}
+        if(strcmp(argv[1],"oftableinit1") == 0) {
+		radeon_of_table(argv[2], OF_TABLE_INIT1);
+		return 0;
+	}
+        if(strcmp(argv[1],"oftableinit2") == 0) {
+		radeon_of_table(argv[2], OF_TABLE_INIT2);
+		return 0;
+	}
+        if(strcmp(argv[1],"oftablememinit") == 0) {
+		radeon_of_table(argv[2], OF_TABLE_MEMINIT);
+		return 0;
+	}
+    	map_radeon_cntl_mem();
         if(strcmp(argv[1],"dac") == 0) {
             radeon_cmd_dac(argv[2]);
             return 0;
@@ -2922,15 +3043,15 @@ int main(int argc,char *argv[])
         } else if(strcmp(argv[1],"regmatch") == 0) {
             radeon_reg_match(argv[2]);
             return 0;
-        } else if(strcmp(argv[1],"romtables") == 0) {
-		radeon_rom_tables(argv[2]);
-		return 0;
-	};
+	}
     } else if(argc == 4) {
-	    if(strcmp(argv[1],"regset") == 0) {
-		    radeon_reg_set(argv[2], strtoul(argv[3], NULL, 0));
-		    return 0;
-	    }
+    	map_radeon_cntl_mem();
+	if(strcmp(argv[1],"regset") == 0) {
+	    radeon_reg_set(argv[2], strtoul(argv[3], NULL, 0));
+	    return 0;
+	}
+    } else {
+	map_radeon_cntl_mem();
     }
 
     usage();
